"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Requestly_1 = require("./services/Requestly");
class GmailTransport {
    constructor(options) {
        this.options = options;
        this.name = 'GmailTransport';
        this.version = 'N/A';
    }
    messageToBase64Raw(message) {
        return message.toString('base64')
            .replace(/\+/g, '-')
            .replace(/\//g, '_')
            .replace(/=+$/, '');
    }
    refreshTokenParams(auth) {
        return {
            client_id: auth.clientId,
            client_secret: auth.clientSecret,
            refresh_token: auth.refreshToken,
            grant_type: 'refresh_token'
        };
    }
    createError(ge) {
        if (ge.error_description) {
            return new Error(ge.error_description);
        }
        if (typeof ge.error === 'object' && ge.error !== null) {
            return new Error(ge.error.message);
        }
        return new Error((ge.error || ge));
    }
    getAccessToken() {
        return Requestly_1.Requestly.post({
            protocol: 'https:',
            hostname: 'www.googleapis.com',
            path: '/oauth2/v4/token'
        }, this.refreshTokenParams(this.options.auth))
            .then((data) => data.access_token);
    }
    sendMail(message, accessToken) {
        return Requestly_1.Requestly.postJSON({
            protocol: 'https:',
            hostname: 'www.googleapis.com',
            path: `/gmail/v1/users/${this.options.userId || 'me'}/messages/send`,
            headers: {
                Authorization: `Bearer ${accessToken}`
            }
        }, {
            raw: this.messageToBase64Raw(message)
        });
    }
    send(mail, done) {
        mail.message.keepBcc = true;
        mail.message.build((error, data) => {
            if (error)
                return done(error);
            this.sendMail(data, this.options.auth.accessToken).then((message) => {
                done(null, {
                    envelope: mail.message.getEnvelope(),
                    messageId: mail.message.messageId(),
                    accessToken: this.options.auth.accessToken,
                    message: message
                });
            }, (e) => {
                if (e.error.code === 401 && this.options.auth.refreshToken) {
                    this.getAccessToken().then((accessToken) => {
                        this.sendMail(data, accessToken).then((message) => {
                            done(null, {
                                envelope: mail.message.getEnvelope(),
                                messageId: mail.message.messageId(),
                                accessToken: accessToken,
                                message: message
                            });
                        }).catch((er) => done(this.createError(er)));
                    }).catch((er) => done(this.createError(er)));
                }
                else {
                    done(this.createError(e));
                }
            });
        });
    }
}
exports.GmailTransport = GmailTransport;
exports.default = GmailTransport;
