"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const https_1 = require("https");
var Requestly;
(function (Requestly) {
    function parseJSON(rawData) {
        try {
            return JSON.parse(rawData);
        }
        catch (ignore) {
            return rawData;
        }
    }
    function makeParams(params) {
        return Object.keys(params)
            .map(k => `${encodeURIComponent(k)}=${encodeURIComponent(params[k])}`).join('&');
    }
    function sendRequest(options, params) {
        return new Promise((resolve, reject) => {
            let req = https_1.request(options, (res) => {
                let chunks = [];
                res.on('data', (chunk) => chunks.push(chunk));
                res.on('end', () => {
                    let answer = parseJSON(Buffer.concat(chunks).toString());
                    if (res.statusCode === 200) {
                        resolve(answer);
                    }
                    else {
                        reject(answer);
                    }
                });
                res.on('error', (error) => {
                    reject(error);
                });
            });
            req.on('error', (error) => {
                reject(error);
            });
            if (params) {
                req.write(params);
            }
        });
    }
    function post(options, params) {
        let postData = makeParams(params);
        options.method = 'POST';
        options.headers = {
            'Content-Type': 'application/x-www-form-urlencoded;charset=UTF-8',
            'Content-Length': Buffer.byteLength(postData)
        };
        return sendRequest(options, postData);
    }
    Requestly.post = post;
    function postJSON(options, data) {
        let json = JSON.stringify(data);
        options.method = 'POST';
        options.headers = Object.assign(options.headers || {}, {
            'Content-Type': 'application/json',
            'Content-Length': Buffer.byteLength(json)
        });
        return sendRequest(options, json);
    }
    Requestly.postJSON = postJSON;
})(Requestly = exports.Requestly || (exports.Requestly = {}));
